package info.extensiblecatalog.nacm.gui;

import info.extensiblecatalog.nacm.gui.scheduling.ImportTaskPanel;
import info.extensiblecatalog.nacm.gui.scheduling.Job;
import info.extensiblecatalog.nacm.gui.scheduling.MatchTaskPanel;
import info.extensiblecatalog.nacm.importer.RecordImporter;
import info.extensiblecatalog.nacm.matcher.MatchDriver;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DateFormat;
import java.util.LinkedList;
import java.util.List;
import java.util.Timer;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

/**
 * @author Rob
 *
 */
@SuppressWarnings("serial")
public class JobScheduler extends JPanel {
	
	public static final DateFormat DATE_FORMAT = DateFormat.getDateTimeInstance( DateFormat.MEDIUM, DateFormat.SHORT );
	
	public static final String[] COLUMN_NAMES = { "#", "Job Name", "Scheduled", "Progress" };

	private JobTableModel tableModel = new JobTableModel();
	private JTable table = new JTable( tableModel );
	private Timer timer;
	
	// other packages this needs to make calls to
	private RecordImporter importer;
	private MatchDriver matcher;
	
	// GUI components that need to be programtically affected at runtime
	private JSplitPane splitPane;
	private JTextArea errorLogArea;
	private JButton toggleErrorLogButton;
	private JLabel unreadErrorCountLabel;
	private JButton clearOldJobsButton;
	private JButton clearErrorLogButton;
	
	private int unreadErrorCount = 0;

	
	public JobScheduler( RecordImporter importer, MatchDriver matcher ) {
		super( new BorderLayout() );
		
		JPanel errorPanel = createErrorPanel();
		splitPane = wrapInSplitPane( table, errorPanel );
		
		this.add( createUpperPanel(), BorderLayout.NORTH );
		this.add( splitPane, BorderLayout.CENTER );
//		this.add( new JScrollPane( table ), BorderLayout.CENTER );
		this.add( createClearOldJobsButton(), BorderLayout.SOUTH );
		
		setColumnWidthsToDefaults();
		setProgressBarRenderer();
		
		timer = new Timer("NACM Job Scheduler");
		this.importer = importer;
		this.matcher = matcher;
	}
	
	private JSplitPane wrapInSplitPane( JTable table, JComponent errorPanel ) {
		JSplitPane splitPane = new JSplitPane( JSplitPane.VERTICAL_SPLIT );
		splitPane.setTopComponent( new JScrollPane( table ) );
		splitPane.setBottomComponent( errorPanel );
		splitPane.setResizeWeight( 0.4 );
//		splitPane.setDividerLocation( 1.0f );	// why doesn't this work?
		splitPane.setOneTouchExpandable( false );
		splitPane.setDividerSize( 7 );	// default is 10 pixels
		
		return splitPane;
	}
	
	private JPanel createErrorPanel() {
		JPanel panel = new JPanel( new BorderLayout() );
		
		errorLogArea = new JTextArea();
		errorLogArea.setEditable( false );
		
		panel.add( new JLabel("Error log:"), BorderLayout.NORTH );
		panel.add( new JScrollPane( errorLogArea ), BorderLayout.CENTER );
		
		return panel;
	}

	private JComponent createUpperPanel() {
		JPanel panel = new JPanel();
		BoxLayout layout = new BoxLayout( panel, BoxLayout.Y_AXIS );
		panel.setLayout( layout );
		
		JPanel disclaimerPanel = new JPanel();
		disclaimerPanel.add( new JLabel("This program must be running for scheduled jobs to occur.") );
		
		panel.add( createButtons() );
		panel.add( disclaimerPanel );
		
		return panel;
	}
	
	private JComponent createButtons() {
		JPanel panel = new JPanel();
		JButton importButton = new JButton("Import");
		importButton.setToolTipText("Creates and schedules a job to import records from a file or directory.");
		JButton matchButton = new JButton("Match");
		matchButton.setToolTipText("Creates and schedules a job to create links between previously imported Bib records to Auth records.");
		
		importButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				JobDialog d = new JobDialog( new ImportTaskPanel( importer ), JobScheduler.this );
				d.setVisible( true );
			}
		});
		matchButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				JobDialog d = new JobDialog( new MatchTaskPanel( matcher ), JobScheduler.this );
				d.setVisible( true );
			}
		});
		
		panel.add( importButton );
		panel.add( matchButton );
		return panel;
	}
	
	private JComponent createClearOldJobsButton() {
		
		toggleErrorLogButton = new JButton("Show Error Log");
		toggleErrorLogButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				if ( errorLogIsVisible() ) {
					// hide error log
					splitPane.setDividerLocation( 1.0 );
				} else {
					// show error log
					splitPane.setDividerLocation( splitPane.getLastDividerLocation() );
				}
			}
		});
		splitPane.addPropertyChangeListener( new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if ( e.getPropertyName().equals( JSplitPane.DIVIDER_LOCATION_PROPERTY ) ) {
					if ( errorLogIsVisible() ) {
						unreadErrorCount = 0;
						unreadErrorCountLabel.setText("");
						toggleErrorLogButton.setText("Hide Error Log");
					} else {
						toggleErrorLogButton.setText("Show Error Log");
					}
				}
			}
		});
		
		clearOldJobsButton = new JButton("Clear Completed Jobs");
		clearOldJobsButton.setEnabled( false );
		clearOldJobsButton.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tableModel.clearOldJobs();
				clearOldJobsButton.setEnabled( false );
			}
		});
		
		clearErrorLogButton = new JButton("Clear Error Log");
		clearErrorLogButton.setEnabled( false );
		clearErrorLogButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				if ( !errorLogIsVisible() && unreadErrorCount > 0 ) {
					int choice = JOptionPane.showConfirmDialog( null, "You have not read " + unreadErrorCount +
							" of the error messages, are you sure you want to clear them all?",
							"Title", JOptionPane.YES_NO_OPTION);
					if ( choice == JOptionPane.NO_OPTION ) {
						return;
					}
				}
				clearErrorLog();
			}
		});
		
		// create without text, because there are no error messages yet
		// the text will be set when addErrorMesssage() is called
		unreadErrorCountLabel = new JLabel("");
		unreadErrorCountLabel.setForeground( Color.RED );
		

		JPanel panel = new JPanel( new BorderLayout() );
		JPanel rightPanel = new JPanel();
		JPanel leftPanel = new JPanel();

		leftPanel.add( toggleErrorLogButton );
		rightPanel.add( clearOldJobsButton );
		rightPanel.add( clearErrorLogButton );
		panel.add( leftPanel, BorderLayout.WEST );
		panel.add( unreadErrorCountLabel, BorderLayout.CENTER );
		panel.add( rightPanel, BorderLayout.EAST );
		
		return panel;
	}
	
	public void setColumnWidthsToDefaults() {
		table.getColumn( COLUMN_NAMES[0] ).setPreferredWidth( 30 );
		table.getColumn( COLUMN_NAMES[1] ).setPreferredWidth( 300 );
		table.getColumn( COLUMN_NAMES[2] ).setPreferredWidth( 250 );
		table.getColumn( COLUMN_NAMES[3] ).setPreferredWidth( 250 );
	}
	
	private void setProgressBarRenderer() {
		TableColumn tc = table.getColumn("Progress");
		tc.setCellRenderer( new DefaultTableCellRenderer() {
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				if ( value instanceof JProgressBar ) { 
					return (JProgressBar)value;
				} else {
					return super.getTableCellRendererComponent( table, value, isSelected, hasFocus, row, column );
				}
			}
		});
	}

	public Timer getTimer() {
		return timer;
	}
	
	public void addJob( Job j ) {
		tableModel.addJob( j );
		
		// allow the job to schedule itself
		j.uponAddingToScheduler( this );
	}
	
	public void updateTable() {
		tableModel.fireTableDataChanged();
	}
	
	public int indexOf( Job j ) {
		return tableModel.jobList.indexOf( j );
	}
	
	static class JobTableModel extends AbstractTableModel {

		private List<Job> jobList = new LinkedList<Job>();
		
		public void addJob( Job j ) {
			jobList.add( j );
			fireTableDataChanged();
		}
		
		public void clearOldJobs() {
			for ( int i = 0; i < jobList.size(); i++ ) {
				Job j = jobList.get( i );
				if ( j.isCompleted() ) {
					jobList.remove( i );
					i--;	// since removing an item shifted the list down by one
				}
			}
			fireTableDataChanged();
		}

//		public void removeJob( Job j ) {
//			jobList.remove( j );
//			fireTableDataChanged();
//		}
		
		public int getRowCount() {
			return jobList.size();
		}

		public int getColumnCount() {
			return COLUMN_NAMES.length;
		}
		
		public String getColumnName( int columnIndex ) {
			return COLUMN_NAMES[ columnIndex ];
		}
		
		public Class<?> getClassForColumn( int columnIndex ) {
			if ( columnIndex == 3 ) {
				return JProgressBar.class;
			} else {
				return String.class;
			}
		}

		public Object getValueAt(int row, int column) {
			Job rowData = jobList.get( row );
			Object retVal = null;
			
			switch( column ) {
			case 0:
				retVal = "" + rowData.getID();
				break;
			case 1:
				retVal = rowData.getJobName();
				break;
			case 2:
				retVal = rowData.getScheduleDescription();
				break;
			case 3:
				retVal = rowData.getProgress();
				break;
			}
			
			return retVal;
		}
		
	} // class JobTableModel

	public Vector<Job> getUncompletedJobs() {
		Vector<Job> list = new Vector<Job>();
		
		for ( Job j : tableModel.jobList ) {
			if ( !j.isCompleted() ) {
				list.add( j );
			}
		}
		
		return list;
	}
	
	public void setUncompletedJobsExist( boolean uncompletedJobs ) {
		clearOldJobsButton.setEnabled( uncompletedJobs );
	}
	
	public void addErrorMessage( String errorMessage ) {
		errorLogArea.append( errorMessage + "\n" );
		clearErrorLogButton.setEnabled( true );
		
		if ( !errorLogIsVisible() ) {
			// increase unread message count
			unreadErrorCount++;
			
			// update JLabel
			if ( unreadErrorCount == 1 ) {
				unreadErrorCountLabel.setText("There is 1 unread error message." );
			} else {
				unreadErrorCountLabel.setText("There are " + unreadErrorCount + " unread error messages." );
			}
		}
	}

	private boolean errorLogIsVisible() {
		return splitPane.getDividerLocation() < splitPane.getMaximumDividerLocation();
	}
	
	public void hideErrorLog() {
		splitPane.setDividerLocation( 1.0 );
	}
	
	private void clearErrorLog() {
		errorLogArea.setText("");
		unreadErrorCount = 0;
		clearErrorLogButton.setEnabled( false );
		unreadErrorCountLabel.setText("");
	}

	
} // class JobScheduler
