package info.extensiblecatalog.nacm.gui;

import info.extensiblecatalog.nacm.model.AuthorityLink;
import info.extensiblecatalog.nacm.model.persistence.AuthorityLinkDAO;
import info.extensiblecatalog.nacm.model.persistence.Filter;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableColumnModelListener;
import javax.swing.table.AbstractTableModel;

/**
 * @author Rob Busack
 *
 * Just one part of the NACM Client GUI.
 */
@SuppressWarnings("serial")
public class ResultsTable extends JPanel {
	
	public static final DateFormat DATE_FORMAT = DateFormat.getDateTimeInstance( DateFormat.MEDIUM, DateFormat.SHORT );

	public static final String[] COLUMN_NAMES = new String[ Filter.NUM_FIELDS ];
	{
		COLUMN_NAMES[ Filter.FLAGGED ] 	= "Flagged";
		COLUMN_NAMES[ Filter.MARKED_AS ]	= "Marked As";
		COLUMN_NAMES[ Filter.BIB_NAME ]	= "Bibliographic Name";
		COLUMN_NAMES[ Filter.AUTH_NAME ]	= "Authoritative Name";
		COLUMN_NAMES[ Filter.CONFIDENCE ]	= "Confidence";
		COLUMN_NAMES[ Filter.REASON ]	= "Reason for Match";
		COLUMN_NAMES[ Filter.DATE_CREATED ]	= "Date Found";
		COLUMN_NAMES[ Filter.MARKED_BY ]	= "Marked By";
	};
	
	public static final boolean RELEASE_1 = true;
	
	private final ResultsTableModel tableModel = new ResultsTableModel();
	private final JTable table = new JTable( tableModel );
	
	private AuthorityLinkDAO database;
	private FilterSettingsPanel filterSettingsPanel;
	private JPanel filterSettingsPanelContainer;
	private LinkControls linkEditingControls;
	
	
	public ResultsTable( AuthorityLinkDAO database ) {
		this.database = database;
		
		setColumnWidthsToDefaults();
		makeSortableByColumn();
		
		filterSettingsPanel = new FilterSettingsPanel( this );
		filterSettingsPanelContainer = new JPanel( new BorderLayout() );
		filterSettingsPanelContainer.add( createRefreshButton(), BorderLayout.SOUTH  );
		
		// put together the components of the GUI
		JScrollPane tableInScrollPane = new JScrollPane(table);
		this.setLayout( new BorderLayout() );
		this.add( filterSettingsPanelContainer, BorderLayout.NORTH );
		this.add( tableInScrollPane, BorderLayout.CENTER );
		this.add( createLinkEditingPanel(), BorderLayout.SOUTH );
	}
	
	private void makeSortableByColumn() {
		table.getColumnModel().addColumnModelListener( new TableColumnModelListener() {
			public void columnAdded(TableColumnModelEvent e) {}
			public void columnRemoved(TableColumnModelEvent e) {}
			public void columnMoved(TableColumnModelEvent e) {}
			public void columnMarginChanged(ChangeEvent e) {}
			public void columnSelectionChanged(ListSelectionEvent e) {
				// don't try to sort by a column while the selected column is being changed
				if ( !e.getValueIsAdjusting() ) {
					sortBy( e.getFirstIndex() );
				}
			}
		});
	}
	
	/**
	 * Assumes table has already been instantiated.
	 * 
	 * @return
	 */
	private JComponent createRefreshButton() {
		JButton refreshButton = new JButton( "Refresh" );
		
		refreshButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				refresh();
			}
		});
		
		final JButton toggleFilterPanelButton = new JButton("Show Filter Options");
		toggleFilterPanelButton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				if ( filterSettingsPanel.getParent() == null ) {
					filterSettingsPanelContainer.add( filterSettingsPanel, BorderLayout.CENTER );
					toggleFilterPanelButton.setText("Hide Filter Options");
				} else {
					filterSettingsPanelContainer.remove( filterSettingsPanel );
					toggleFilterPanelButton.setText("Show Filter Options");
				}
				filterSettingsPanelContainer.validate();
			}
		});
	
		// by wrapping the button in an extra JPanel with FlowLayout,
		// the wrapperPanel get's stretched by the enclosing BorderLayout
		// instead of the button, so you don't get an oddly huge button
		JPanel wrapperPanel = new JPanel();
		if ( !RELEASE_1 ) {
			wrapperPanel.add( toggleFilterPanelButton );
		}
		wrapperPanel.add( refreshButton );
		return wrapperPanel;
	}
	
	private JComponent createLinkEditingPanel() {
		linkEditingControls = new LinkControls( database, this );
		
		table.getSelectionModel().addListSelectionListener( new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				List<AuthorityLink> selectedLinks = tableModel.getLinksAt( table.getSelectedRows() );
				linkEditingControls.setSelection( selectedLinks );
			}
		});
		
		return linkEditingControls;
	}
	
	/**
	 * Pulls the latest data out of the database and shows it in
	 * the results table.
	 *
	 */
	public void refresh() {
		// remember the selection
//		int[] selectedIDs = tableModel.indexsToIDs( table.getSelectedRows() );
		int[] selectedRows = table.getSelectedRows();
		
		List<AuthorityLink> data = null;
		
		if ( RELEASE_1 ) {
			data = database.findAll();
		} else {
			data = database.findAllWithFilter( filterSettingsPanel.getFilter() );
		}
		
		
		if ( data != null ) {
			tableModel.setData( data );
			
//			int[] selectedIndexes = tableModel.idsToIndexes( selectedIDs );
//			for ( int selectedRow : selectedIndexes ) {
//				table.getSelectionModel().addSelectionInterval( selectedRow, selectedRow );
//			}
			for ( int index : selectedRows ) {
				table.addRowSelectionInterval( index, index );
			}
		} else {
			JOptionPane.showMessageDialog( this, 
					"The refresh failed.  The data displayed in table will be left unchanged and may be out of date.\n\n" +
					"Technical reason:\n    A null pointer was returned by the class AuthorityLinkDAO instead of a new list of links.");
		}
	}
	
	private void sortBy( int columnIndex ) {
		Filter f = filterSettingsPanel.getFilter();
		f.setSortBy( columnIndex );
	}
	
	public void setColumnWidthsToDefaults() {
		table.getColumn( COLUMN_NAMES[Filter.FLAGGED] ).setPreferredWidth( 40 );
		table.getColumn( COLUMN_NAMES[Filter.MARKED_AS] ).setPreferredWidth( 60 );
		table.getColumn( COLUMN_NAMES[Filter.BIB_NAME] ).setPreferredWidth( 110 );
		table.getColumn( COLUMN_NAMES[Filter.AUTH_NAME] ).setPreferredWidth( 110 );
		table.getColumn( COLUMN_NAMES[Filter.CONFIDENCE] ).setPreferredWidth( 70 );
		table.getColumn( COLUMN_NAMES[Filter.REASON] ).setPreferredWidth( 100 );
		table.getColumn( COLUMN_NAMES[Filter.DATE_CREATED] ).setPreferredWidth( 130 );
		table.getColumn( COLUMN_NAMES[Filter.MARKED_BY] ).setPreferredWidth( 80 );
	}
	
	private class ResultsTableModel extends AbstractTableModel {
		
		private List<AuthorityLink> data = new LinkedList<AuthorityLink>();
		
		public void setData( List<AuthorityLink> data ) {
			assert( data != null );
			
			this.data = data;
			fireTableDataChanged();
		}
		
//		public long[] indexsToIDs(int[] selectedRows) {
//			long[] ids = new long[ selectedRows.length ];
//			for ( int i : selectedRows ) {
//				ids[i] = data.get(i).getId();
//			}
//			return ids;
//		}
//
//		public int[] idsToIndexes(int[] selectedIDs) {
//			long[] indexes = new long[ selectedIDs.length ];
//			for ( int i : selectedIDs ) {
//				indexes[i] = data.indexOf()
//			}
//			return ids;
//		}

		public List<AuthorityLink> getLinksAt( int[] indexes ) {
			List<AuthorityLink> selectedLinks = new LinkedList<AuthorityLink>();
			for ( int i : indexes ) {
				selectedLinks.add( data.get( i ) );
			}
			return selectedLinks;
		}

		public Object getValueAt( int row, int column ) {
			AuthorityLink rowData = data.get( row );
			String cellContent = null;
			
			try {
				switch ( column ) {
				case Filter.FLAGGED:
					//cellContent = rowData.isFlagged().toString();
					if ( rowData.isFlagged() ) {
						cellContent = " *";
					} else {
						cellContent = "";
					}
					break;
				case Filter.MARKED_AS:
					if ( rowData.getStatus().equals(AuthorityLink.MARKED_ACCEPTED) ) {
						cellContent = "approved";
					} else if ( rowData.getStatus().equals(AuthorityLink.MARKED_REJECTED) ) {
						cellContent = "rejected";
					} else {
						cellContent = "unverified";
					}
					break;
				case Filter.BIB_NAME:
					// TODO is this right?
					cellContent = rowData.getRelatedName().getOriginalForm();
					break;
				case Filter.AUTH_NAME:
					cellContent = rowData.getRecord().getAuthorizedForm().getOriginalForm();
					break;
				case Filter.CONFIDENCE:
					cellContent = Integer.toString( rowData.getPercentConfidence() ) + "%";
					break;
				case Filter.REASON:
					cellContent = rowData.getEvidence();
					break;
				case Filter.DATE_CREATED:
					cellContent = DATE_FORMAT.format( rowData.getWhenFound() );
					break;
				case Filter.MARKED_BY:
					cellContent = rowData.getApprovedBy();
					break;
				}
			} catch ( NullPointerException e ) {
				cellContent = "NullPointerException";
				// TODO how do I print the stack trace to a debugging log?
				e.printStackTrace();
			}
			return cellContent;
		}
		
		public int getRowCount() {
			return data.size();
		}
		
		public int getColumnCount() {
			return Filter.NUM_FIELDS;
		}
	
		public String getColumnName( int columnIndex ) {
			return COLUMN_NAMES[ columnIndex ];
		}
		
		public boolean isCellEditable( int row, int column ) {
			return false;
		}
		
	} // class ResultsTableModel

}
