package info.extensiblecatalog.nacm.gui.scheduling;

import info.extensiblecatalog.nacm.gui.JobScheduler;

import java.util.Date;
import java.util.Set;
import java.util.TimerTask;
import java.util.TreeSet;

public class Job {
	
	public static final int IMMEDIATELY = 0;
	public static final int AT_DATE_TIME = 1;
	public static final int AFTER_OTHER_JOB = 2;
	
//	private static final long DELETE_AFTER_COMPLETION_DELAY = 7000; // milliseconds

	private int scheduledFor;
	
	private Task task;
	
	private Date startTime;
//	private Job prerequisite;
	private Set<Job> waitingJobs = new TreeSet<Job>();
	
	private boolean hasStarted = false;
	private boolean hasCompleted = false;
	
	/**
	 * A String for the GUI, describing when this job is scheduled to run.
	 */
	private String scheduleDescription;
	private JobScheduler jobList;
	
	
	public Job( JobScheduler jobList, Task task ) {
		this.jobList = jobList;
		this.task = task;
		task.setOwnerJob( this );
		
		scheduleDescription = "Immediately";
		scheduledFor = IMMEDIATELY;
	}
	
	public Job( JobScheduler jobList, Task task, Date startTime ) {
		this ( jobList, task );
		this.startTime = startTime;
		scheduleDescription = JobScheduler.DATE_FORMAT.format( startTime );
		scheduledFor = AT_DATE_TIME;
	}
	
	public Job( JobScheduler jobList, Task task, Job prerequisite ) {
		this( jobList, task );

//		this.prerequisite = prerequisite;
		prerequisite.addWaitingJob( this );
		scheduleDescription = "On completion of Job " + prerequisite.getID();
		scheduledFor = AFTER_OTHER_JOB;
	}

	public int getID() {
		return jobList.indexOf( this ) + 1;
	}
	
	public String getJobName() {
		return task.getTaskName();
	}
	
	public String getScheduleDescription() {
		String retVal = null;
		if ( !hasStarted ) {
			retVal = scheduleDescription;
		} else {
			retVal = scheduleDescription + ": " + task.getProgressString();
		}
		return retVal;
	}
	
	public Object getProgress() {
		if ( hasStarted && !hasCompleted ) {
			return task.getProgressBar();
		} else {
			return " - ";
		}
	}

	private void addWaitingJob(Job job) {
		waitingJobs.add( job );		
	}

	public void uponAddingToScheduler( JobScheduler scheduler ) {
		if ( scheduledFor == AT_DATE_TIME ) {
			TimerTask timerTask = new TimerTask() {
				public void run() {
					start();
				}
			};
			scheduler.getTimer().schedule( timerTask, startTime );
		} else if ( scheduledFor == IMMEDIATELY ) {
			start();
		}
	}

	public void start() {
		hasStarted = true;
		scheduleDescription = "Running now";
		Thread t = new Thread( task );
		t.start();
	}
	
	public void setCompleteSuccessfully() {
		// make the visual changes to indicate that this job has completed
		scheduleDescription = "Completed";

		setCompletedPassOrFail();
		
//		// wait a few seconds so the user can see this completed, then make it go away
//		try {
//			Thread.sleep( DELETE_AFTER_COMPLETION_DELAY );
//		} catch (InterruptedException e) { } // all that is lost is a little user friendliness
//	
//		jobList.removeJob( this );
	}
	
	public boolean isCompleted() {
		return hasCompleted;
	}

	public void updateGUI() {
		jobList.updateTable();
	}

	public void noteFailure( String source, Exception reason ) {
		// Format: "<Source> - <ExceptionClass>: 
			
		// rather than consider a failure a completion, just note it and continue
		String errorMessage = "";

		String exceptionClassName = reason.getClass().getSimpleName();
		errorMessage += source + " - " + exceptionClassName + ": ";
		errorMessage += reason.getMessage();
		errorMessage += "     (occured during '" + getJobName() + "', ";
		errorMessage += JobScheduler.DATE_FORMAT.format( new Date() );
		errorMessage += ")";
		
		jobList.addErrorMessage( errorMessage );

		
		
		
		
		
//		String veryShortDescription = exceptionClassName;
		
		// TODO proper logging
		System.err.println( reason.getMessage() );
		reason.printStackTrace();
		
			
		// stuff that will show in the GUI	
//		scheduleDescription = "Failed: " + veryShortDescription;
//
//		setCompletedPassOrFail();
	}
	
	private void setCompletedPassOrFail() {
		hasCompleted = true;
		updateGUI();
		
		// start any jobs that are waiting on this one
		for ( Job j : waitingJobs ) {
			j.start();
		}
		
		jobList.setUncompletedJobsExist( true );
	}
	
	public String toString() {
		return "#" + getID() + ". " + getJobName();
	}
	
}
