package info.extensiblecatalog.nacm.matcher;

import info.extensiblecatalog.nacm.model.AuthorityLink;
import info.extensiblecatalog.nacm.model.BibliographicRecord;
import info.extensiblecatalog.nacm.model.Name;
import info.extensiblecatalog.nacm.model.persistence.BibliographicRecordDAO;
import info.extensiblecatalog.nacm.model.persistence.DataAccessException;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Attempt to match all unmatched Bibliographic records
 * @author Ben Greenwood
 *
 */
public class MatchDriver{

	List<MatchStrategy> match_strats;
	BibliographicRecordDAO bibDAO;
	int total_strats;
	
	/**
	 * Initalize the MatchDriver, this creates the match strategy list and the bibDAO
	 */
	public MatchDriver(){
		
		bibDAO=new BibliographicRecordDAO();
		
		match_strats = new ArrayList<MatchStrategy>(1); // only 1 strat for release 1
		
		// create our list of strategies in the order we want to use them
		match_strats.add(new MatchAuthorizedForm());
		
		total_strats=match_strats.size();
	}

	/**
	 * Attemps to match all unmatched bib records to auth records using
	 * the list of matching strategies.
	 */
	public void match() {
		match(new ProgressCallback());
	}

	/**
	 * Attemps to match all unmatched bib records to auth records using
	 * the list of matching strategies. A progress callback is provided
	 * so the calling program can keep tabs on the matching's overall progress
	 * @param progress - a callback object used to track the matching progress
	 */
	public void match( ProgressCallback progress){
		
		List<BibliographicRecord> unmatched = bibDAO.findAllUnmatched();
		int total_found = unmatched.size();
		progress.setLengthOfTask(total_found);
		int bib_number=1;
		for ( BibliographicRecord curr_bib: unmatched ){
			Set<AuthorityLink> auth_links = new HashSet<AuthorityLink>();
			Set<Name> authors = curr_bib.getAuthors();
			for (Name curr_auth : authors ){
				auth_links.addAll(findAuthMatch(curr_auth));
			}
			try
			{
				storeMatchedBib(curr_bib,auth_links);
			}
			catch(DataAccessException e)
			{
				progress.itemFailed(curr_bib, e);
			}
			
			progress.setProgress(bib_number);
			bib_number++;
		}
	}
	
	/**
	 * Attempt to match a single bib record to auth records using 
	 * the list of matching strategies. The matching stops after a
	 * strategy returns 1 or more results.
	 * @param bib the bib record to find authority record matches for.
	 * @return	a set of authority record links
	 */
	Set<AuthorityLink> findAuthMatch(Name curr_auth){
		
		Set<AuthorityLink> links=null;
		for ( MatchStrategy current_strat: match_strats ){
			links = current_strat.performMatching(curr_auth);
			if ( links.size()>=1 ){ // matches is null until after the 1st match strat
				break;
			}
		}
		return links;
	}
	
	/**
	 * Updates the bib record and stores the authority links for it in the database.
	 * @param bib the bib record to update
	 * @param auth_links the set of associated authority links to store with the bib
	 */
	void storeMatchedBib(BibliographicRecord bib, Set<AuthorityLink> auth_links)
	throws DataAccessException
	{
		// only in release 1 can a bib record fail to be matched
		if ( auth_links.size() > 0 ){
			bib.setMatched(true);
			//bib.setAuthorizedAuthors(auth_links);
			for ( AuthorityLink al : auth_links ){
				bib.getAuthorizedAuthors().add(al);
			}
			
			bibDAO.store(bib);
		}
	}
}