package info.extensiblecatalog.nacm.matcher;

import info.extensiblecatalog.nacm.model.*;

import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.Date;

/**
 * 
 * @author Ben Greenwood
 * This abstract class is uses to define the methods that a 
 * match strategy has. All matching strategies will extend
 * this class and implement their own version of performMatching 
 */
abstract class MatchStrategy {
	
	/**
	 * This method must be implemented in classes that extend
	 * MatchStrategy. This method is used to find and create
	 * authority links for all authority records that match
	 * the passed in bib record. 
	 * @param bib the bib record to attempt to match to
	 * authority records
	 * @return the set of created authority links (this set
	 * should be empty if no links were found)
	 */
	abstract public Set<AuthorityLink> performMatching(Name curr_auth);
	
	/**
	 * This method is provided to give other matching strategies
	 * a simple way to create authority links.
	 * @param auth_records the authority records to create links to
	 * @param auth_name the name that was used to match to the auth record
	 * @param confidence an integer (1-100) indicating how certain
	 * the strategy is that it has found a valid match 
	 * @param approved if the confidence is high enough to
	 * automatically approve the record
	 * @param evidence a string telling what evidence was used to make the match
	 * @return a set of authority links created with the information
	 */
	protected Set<AuthorityLink> createAuthLinks( List<AuthorityRecord> auth_records, Name auth_name, int confidence, boolean approved, String evidence){
		Set<AuthorityLink> links = new HashSet<AuthorityLink>();

		Date curr_date = new Date();

		for ( AuthorityRecord curr_auth : auth_records ){
			AuthorityLink curr_link = new AuthorityLink(curr_auth,auth_name,curr_date);
			curr_link.setPercentConfidence(confidence);
			if ( approved){
				curr_link.setStatus(AuthorityLink.MARKED_ACCEPTED);
				curr_link.setApprovedBy("System");
			}
			curr_link.setEvidence(evidence);
			links.add(curr_link);
		}
		return links;
	}
}