package info.extensiblecatalog.nacm.model;

import java.util.Date;

/**
 * Contains information about a match between a bibliographic record
 * and an authorized name that has been identified by the system
 * 
 * @author Dean Rzonca
 */
public class AuthorityLink
{	
	public static final Integer NOT_REVIEWED = 0;
	public static final Integer MARKED_ACCEPTED = 1;
	public static final Integer MARKED_REJECTED = 2;
	
	Long id;
	Integer percentConfidence;
	Integer status;
	String approvedBy;
	String evidence;
	Boolean flagged;
	Date whenFound;
	Date whenApproved;
	
	Name relatedName;
	
	AuthorityRecord record;

	/**
	 * Creates a new, empty AuthorityLink
	 * Note that this does NOT create a viable object,
	 * and is only present for JavaBeans compliance
	 * 
	 * Using this constructor is not recommended
	 */
	public AuthorityLink()
	{
		id = -1l;
		
		percentConfidence = 0;
		status = NOT_REVIEWED;
		flagged= false;
		approvedBy = "none";
		evidence = "none";
		whenFound = new Date();
		whenApproved = new Date(0);		
		relatedName = null;
		record = null;
	}
	
	/**
	 * Creates a new AuthorityLink
	 * @param record		the matched authority record
	 * @param relatedName	the name that was matched
	 * @param whenFound		when the match was found
	 */
	public AuthorityLink(AuthorityRecord record, Name relatedName, Date whenFound)
	{
		id = -1l;
		
		percentConfidence = 0;
		status = NOT_REVIEWED;
		flagged= false;
		approvedBy = "none";
		evidence = "none";
		whenApproved = new Date(0);	
		
		this.whenFound = whenFound;	
		this.relatedName = relatedName;
		this.record = record;		
	}

	/**
	 * Gets the person who approved this match
	 * @return	the name or initials of the user who approved this match
	 */
	public String getApprovedBy()
	{
		return approvedBy;
	}

	/**
	 * Determines whether this link has been flagged for review
	 * @return	true or false
	 */
	public Boolean isFlagged()
	{
		return flagged;
	}

	/**
	 * Sets whether or not this link has been flagged
	 * @param flagged	true or false
	 */
	public void setFlagged(Boolean flagged)
	{
		this.flagged = flagged;
	}

	/**
	 * Determines the approval status of this link
	 * @return	an integer corresponding to a constant
	 */
	public Integer getStatus()
	{
		return status;
	}

	/**
	 * Sets the approval status of this link
	 * @param an integer corresponding to a status constant
	 */
	public void setStatus(Integer status)
	{
		this.status = status;
	}

	/**
	 * Sets the person who approved this match
	 * @param approvedBy	the name or intials of the user who approved this match
	 */
	public void setApprovedBy(String approvedBy)
	{
		this.approvedBy = approvedBy;
	}

	/**
	 * Gets the evidence for this match
	 * @return	a String representation of the evidence for this match
	 */
	public String getEvidence()
	{
		return evidence;
	}

	/**
	 * Sets the evidence for this match
	 * @param evidence	a String representation of the evidence for this match
	 */
	public void setEvidence(String evidence)
	{
		this.evidence = evidence;
	}

	/**
	 * Gets the percent confidence for this match
	 * @return	an integer between 0 and 100
	 */
	public int getPercentConfidence()
	{
		return percentConfidence;
	}

	/**
	 * Sets the percent confidence for this match
	 * @param percentConfidence	an integer between 0 and 100
	 */
	public void setPercentConfidence(int percentConfidence)
	{
		this.percentConfidence = percentConfidence;
	}

	/**
	 * Gets the authority record that has been matched
	 * @return	an AuthorityRecord
	 */
	public AuthorityRecord getRecord()
	{
		return record;
	}

	/**
	 * Sets the authority record that has been matched
	 * @param record	an AuthorityRecord
	 */
	public void setRecord(AuthorityRecord record)
	{
		this.record = record;
	}

	/**
	 * Gets the date when this match was found
	 * @return	a Date
	 */
	public Date getWhenFound()
	{
		return whenFound;
	}

	/**
	 * Sets the date when this match was found
	 * @param whenFound	a Date
	 */
	public void setWhenFound(Date whenFound)
	{
		this.whenFound = whenFound;
	}

	/**
	 * Gets the date when this was approved
	 * @return	a Date, or null if this was not approved
	 */
	public Date getWhenApproved()
	{
		return whenApproved;
	}

	/**
	 * Sets when this was approved
	 * @param whenApproved	a Date
	 */
	public void setWhenApproved(Date whenApproved)
	{
		this.whenApproved = whenApproved;
	}

	/**
	 * Gets the unique ID associated with this record
	 * @return	a unique ID
	 */
	public Long getId()
	{
		return id;
	}

	/**
	 * Sets the unique ID associated with this record
	 * @param id	a unique ID
	 */
	public void setId(Long id)
	{
		this.id = id;
	}

	/**
	 * Gets the original name to which this is related
	 * @return	a Name
	 */
	public Name getRelatedName()
	{
		return relatedName;
	}
	
	/**
	 * Sets the original name to which this is related
	 * @param relatedName	a Name
	 */
	public void setRelatedName(Name relatedName)
	{
		this.relatedName = relatedName;
	}
	
	@Override
	public boolean equals(Object obj)
	{
		boolean equal = false;
		
		if(obj instanceof AuthorityLink)
		{
			AuthorityLink other = (AuthorityLink) obj;
			
			equal = 
				id.equals(other.getId()) &&
				percentConfidence.equals(other.getPercentConfidence()) &&
				status.equals(other.getStatus()) &&
				approvedBy.equals(other.getApprovedBy()) &&
				evidence.equals(other.getEvidence()) &&
				flagged.equals(other.isFlagged()) &&
				(whenFound == other.whenFound || whenFound.equals(other.getWhenFound())) &&
				(relatedName == other.relatedName || relatedName.equals(other.getRelatedName())) &&
				(record == other.record || record.equals(other.getRecord()));
		}
		
		return equal;
	}

	@Override
	public int hashCode()
	{
		return
			id.hashCode() +
			percentConfidence.hashCode() +
			status.hashCode() +
			approvedBy.hashCode() +
			evidence.hashCode() + 
			whenFound.hashCode() +
			whenApproved.hashCode() +
			flagged.hashCode();
	}

	@Override
	public String toString()
	{
		StringBuffer representation = new StringBuffer();
		
		representation.append(percentConfidence);
		representation.append("% probability of match; found on ");
		representation.append(whenFound);
		
		return representation.toString();
	}
	
	
}


