package info.extensiblecatalog.nacm.model;

import java.util.Set;
import java.util.HashSet;

/**
 * Represents the necessary fields for an authority record.
 *  
 * @author Dean Rzonca
 */
public class AuthorityRecord
{
	public static final String MARC = "marc";
	
	Long id;
	Name authorizedForm;
	Set<Name> alternateForms;
	Set<Name> seeAlso;
	String originalXML;
	Boolean generated;
	Integer xmlHash;
	String recordType;

	/**
	 * Creates a new, empty AuthorityRecord
	 * Note that this constructor is only present
	 * for JavaBeans compliance and should not be used
	 */
	public AuthorityRecord()
	{
		id = -1l;
		
		authorizedForm = new Name();
		alternateForms = new HashSet<Name>();
		seeAlso = new HashSet<Name>();
		originalXML = "";
		generated = false;
		recordType = MARC;
		xmlHash = 0;
	}
	
	/**
	 * Creates a new AuthorityRecord
	 * @param authorizedForm	the authorized form of the name
	 * @param originalXML		the original record XML
	 * @param xmlHash			the hash code of the XML
	 * @param generated			whether or not this is a generated record
	 */
	public AuthorityRecord(Name authorizedForm, String originalXML, Integer xmlHash, Boolean generated)
	{
		id = -1l;
		
		alternateForms = new HashSet<Name>();
		seeAlso = new HashSet<Name>();
		recordType = MARC;
		
		this.authorizedForm = authorizedForm;
		this.originalXML = originalXML;
		this.generated = generated;
		this.xmlHash = xmlHash;
	}
	
	/**
	 * Gets the authorized form for this name in normalized form
	 * @return	a Name
	 */
	public Name getAuthorizedForm()
	{
		return authorizedForm;
	}
	
	/**
	 * Sets the authorized form for this name in normalized form
	 * @param authorizedForm	
	 */
	public void setAuthorizedForm(Name authorizedForm)
	{
		this.authorizedForm = authorizedForm;
	}
	
	/**
	 * Gets the alternate forms for this name
	 * @return	All known alternate forms for this name
	 */
	public Set<Name> getAlternateForms()
	{
		return alternateForms;
	}
	
	/**
	 * Sets the alternate forms for this name
	 * @param alternateForms	All known alternate forms for this name
	 */
	public void setAlternateForms(Set<Name> alternateForms)
	{
		this.alternateForms = alternateForms;
	}

	/**
	 * Gets other names which may be related to this record
	 * @return	All known names which may be related to this name, in normalized form
	 */
	public Set<Name> getSeeAlsoReferences()
	{
		return seeAlso;
	}
	
	/**
	 * Sets other names which may be related to this record
	 * @param seeAlsoReferences	All known names which may be related to this name, in normalized form
	 */
	public void setSeeAlsoReferences(Set<Name> seeAlsoReferences)
	{
		this.seeAlso = seeAlsoReferences;
	}
	
	/**
	 * Gets the original, raw XML for this record
	 * @return	the XML that was originally imported
	 */
	public String getOriginalXML()
	{
		return originalXML;
	}
	
	/**
	 * Sets the original source XML for this record
	 * @param xml	the XML data for this record
	 */
	public void setOriginalXML(String xml)
	{
		this.originalXML = xml;
	}
	
	/**
	 * Determines whether this record was imported or generated
	 * @return	true or false
	 */
	public Boolean isGenerated()
	{
		return generated;
	}
	
	/**
	 * Sets whether this record was imported or generated
	 * @param generated	true if the record was generated
	 */
	public void setGenerated(Boolean generated)
	{
		this.generated = generated;
	}

	/**
	 * Gets a hash code for the original XML
	 * Useful for determining whether this record is duplicated
	 * 
	 * The hash code is identical for identical records
	 * @return	a hash code for the XML
	 */
	public Integer getXMLHash()
	{
		return xmlHash;
	}
	
	/**
	 * Sets a hash code for the original XML
	 * This must be identical for identical XML
	 * @param code	a hash code for the original XML
	 */
	public void setXMLHash(Integer code)
	{
		this.xmlHash = code;
	}
	
	/**
	 * Determines the type of record
	 * Use static types defined in this class
	 * @return	a String representing the record type
	 */
	public String getRecordType()
	{
		return recordType;
	}

	/**
	 * Sets the record type
	 * @param recordType	a String representing the record type
	 * 						and corresponding to one of the static
	 * 						items in this class
	 */
	public void setRecordType(String recordType)
	{
		this.recordType = recordType;
	}
	
	/**
	 * Gets the unique ID associated with this record
	 * @return	a unique ID
	 */
	public Long getId()
	{
		return id;
	}

	/**
	 * Sets the unique ID associated with this record
	 * @param id	a unique ID
	 */
	public void setId(Long id)
	{
		this.id = id;
	}

	@Override
	public boolean equals(Object obj)
	{
		boolean equal = false;
		
		if(obj instanceof AuthorityRecord)
		{
			AuthorityRecord other = (AuthorityRecord) obj;
			equal = 
				id.equals(other.getId()) &&
				authorizedForm.equals(other.getAuthorizedForm()) &&
				/*alternateForms.equals(other.getAlternateForms()) &&
				seeAlso.equals(other.getSeeAlsoReferences()) &&*/
				originalXML.equals(other.getOriginalXML()) &&
				generated.equals(other.isGenerated()) &&
				xmlHash.equals(other.getXMLHash());
		}
		
		return equal;
	}

	@Override
	public String toString()
	{
		return authorizedForm.getOriginalForm();
	}

	@Override
	public int hashCode()
	{
		return xmlHash;
	}
	
	
	
	
	
}