package info.extensiblecatalog.nacm.model;

import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;

/**
 * Represents the necessary fields in a bibliographic
 * record.
 * 
 * @author Dean Rzonca
 */
public class BibliographicRecord
{
	Long id;
	
	Set<Name> authors;
	Set<Name> subjects;
	Set<AuthorityLink> authorizedAuthors;
	Set<AuthorityLink> authorizedSubjects;
	Set<Name> titles;
	String originalXML;
	Boolean matched;
	Integer xmlHash;
	
	/**
	 * Creates a new, empty BibliographicRecord
	 * This constructor is only provided for JavaBeans
	 * compliance
	 */
	public BibliographicRecord()
	{
		id = -1l;
		authors = new HashSet<Name>();
		subjects = new HashSet<Name>();
		authorizedAuthors = new HashSet<AuthorityLink>();
		authorizedSubjects = new HashSet<AuthorityLink>();
		titles = new HashSet<Name>();
		originalXML = "";
		matched = false;
		xmlHash = 0;
		
	}
	
	
	public BibliographicRecord(String originalXML, Integer xmlHash)
	{
		id = -1l;
		authors = new HashSet<Name>();
		subjects = new HashSet<Name>();
		authorizedAuthors = new HashSet<AuthorityLink>();
		authorizedSubjects = new HashSet<AuthorityLink>();
		titles = new HashSet<Name>();
		matched = false;
		
		this.originalXML = originalXML;
		this.xmlHash = xmlHash;
		
	}
	
	/**
	 * Gets the original authors for this record, as
	 * they were entered
	 * @return	all authors, as they were entered
	 */
	public Set<Name> getAuthors()
	{
		return authors;
	}
	
	/**
	 * Sets the original authors for this record, as
	 * they were entered
	 * @param authors	all authors, as they were entered
	 */
	public void setAuthors(Set<Name> authors)
	{
		this.authors = authors;
	}
	
	/**
	 * Gets the original subjects for this record, as
	 * they were entered
	 * @return	all subjects, as they were entered
	 */
	public Set<Name> getSubjects()
	{
		return subjects;
	}
	
	/**
	 * Sets the original subjects for this record, as
	 * they were entered
	 * @param authors	all subjects, as they were entered
	 */
	public void setSubjects(Set<Name> subjects)
	{
		this.subjects = subjects;
	}
	
	/**
	 * Gets the authorized authors associated with this record
	 * @return	Information about the authorized authors
	 */
	public Set<AuthorityLink> getAuthorizedAuthors()
	{
		return authorizedAuthors;
	}
	
	/**
	 * Sets the authorized authors associated with this record
	 * @param authors	Information about the authorized authors
	 */
	public void setAuthorizedAuthors(Set<AuthorityLink> authors)
	{
		this.authorizedAuthors = authors;
	}
	
	/**
	 * Gets the authorized subjects associated with this record
	 * @return	Information about the authorized subjects
	 */
	public Set<AuthorityLink> getAuthorizedSubjects()
	{
		return authorizedSubjects;
	}
	
	/**
	 * Sets the authorized subjects associated with this record
	 * @param authors	Information about the authorized subjects
	 */
	public void setAuthorizedSubjects(Set<AuthorityLink> subjects)
	{
		this.authorizedSubjects = subjects;
	}
	
	/**
	 * Gets the titles associated with this record
	 * @return	all titles associated with this record
	 */
	public Set<Name> getTitles()
	{
		return titles;
	}
	
	/**
	 * Sets the titles associated with this record
	 * @param titles	all titles associated with this record
	 */
	public void setTitles(Set<Name> titles)
	{
		this.titles = titles;
	}

	/**
	 * Gets the original, raw XML for this record
	 * @return	the XML that was originally imported
	 */
	public String getOriginalXML()
	{
		return originalXML;
	}
	
	/**
	 * Sets the original source XML for this record
	 * @param xml	the XML data for this record
	 */
	public void setOriginalXML(String xml)
	{
		this.originalXML = xml;
	}
	
	/**
	 * Gets a hash code for the original XML
	 * Useful for determining whether this record is duplicated
	 * 
	 * The hash code is identical for identical records
	 * @return	a hash code for the XML
	 */
	public Integer getXMLHash()
	{
		return xmlHash;
	}
	
	/**
	 * Sets a hash code for the original XML
	 * This must be identical for identical XML
	 * @param code	a hash code for the original XML
	 */
	public void setXMLHash(Integer code)
	{
		this.xmlHash = code;
	}

	/**
	 * Determines if matching has been performed on this record
	 * @return	true if this record has been matched
	 */
	public Boolean isMatched()
	{
		return matched;
	}

	/**
	 * Set whether matching has been performed on this record
	 * @param matched	true if matching has been performed
	 */
	public void setMatched(Boolean matched)
	{
		this.matched = matched;
	}

	/**
	 * Gets the unique ID associated with this record
	 * @return	a unique ID
	 */
	public Long getId()
	{
		return id;
	}

	/**
	 * Sets the unique ID associated with this record
	 * @param id	a unique ID
	 */
	public void setId(Long id)
	{
		this.id = id;
	}

	@Override
	public boolean equals(Object obj)
	{
		boolean equal = false;
		
		if(obj instanceof BibliographicRecord)
		{
			BibliographicRecord other = (BibliographicRecord) obj;
			
			equal = 
				id.equals(other.getId()) &&
				/*authors.equals(other.getAuthors()) &&
				subjects.equals(other.getSubjects()) &&
				authorizedAuthors.equals(other.getAuthorizedAuthors()) &&
				authorizedSubjects.equals(other.getAuthorizedSubjects()) &&
				titles.equals(other.getTitles()) &&*/
				originalXML.equals(other.getOriginalXML()) &&
				matched.equals(other.isMatched()) &&
				xmlHash.equals(other.getXMLHash());
		}
		
		return equal;
	}

	@Override
	public int hashCode()
	{
		return xmlHash;
	}

	@Override
	public String toString()
	{
		StringBuffer titles = new StringBuffer();
		
		Iterator<Name> titleIterator = getTitles().iterator();
		
		while(titleIterator.hasNext())
		{
			Name currentTitle = titleIterator.next();
			titles.append(currentTitle.getOriginalForm());
			
			if(titleIterator.hasNext())
			{
				titles.append(" / ");
			}
		}
		
		return titles.toString();
	}
	
	
}