package info.extensiblecatalog.nacm.model.persistence;

import info.extensiblecatalog.nacm.model.AuthorityRecord;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Provides store/find/delete functionality for
 * authority records
 * @author Dean Rzonca
 *
 */
public class AuthorityRecordDAO
{
	static final String AUTH_CLASS = AuthorityRecord.class.getName();
	
	HibernateHelper helper = new HibernateHelper();
	
	/**
	 * Stores an authority record in the database
	 * If the ID is not set (-1), one will be set
	 * @param recordToStore	the bibliographic record to store
	 * @throws DataAccessException if the record cannot be stored
	 */
	public void store(AuthorityRecord recordToStore)
	throws DataAccessException
	{
		helper.store(recordToStore);
	}
	
	/**
	 * Removes an authority record from the database
	 * @param id	the authority record to remove
	 * 
	 * @throws DataAccessException	if no record exists with the given ID
	 */
	public void remove(AuthorityRecord toRemove)
	throws DataAccessException
	{
		helper.remove(AUTH_CLASS, toRemove);
	}
	
	/**
	 * Finds all authority records that were generated by the NACM
	 * @return		all generated authority records
	 */
	public List<AuthorityRecord> findAllGeneratedRecords()
	{
		return castList(
				helper.loadByBooleanQuery(
						"from " + AUTH_CLASS + " as auth where auth.generated = ?",true));
	}
	
	/**
	 * Finds authority records by their authorized name
	 * @param authorizedName	the authorized name in normalized form
	 * @return					all authority records with the given authorized name
	 */
	public List<AuthorityRecord> findByAuthorizedName(String authorizedName)
	{
		return castList(
				helper.loadByStringQuery(
						"from " + AUTH_CLASS + " as auth where auth.authorizedForm.normalizedForm = ?",authorizedName));
	}
	
	/**
	 * Finds authority records by an alternate name
	 * @param alternateName		the alternate name, in normalized form
	 * @return					all authority records with the given alternate name
	 */
	public List<AuthorityRecord> findByAlternateName(String alternateName)
	{
		return castList(
				helper.loadByStringQuery(
						"from " + AUTH_CLASS + " as auth where auth.alternateForms.normalizedForm = ?",alternateName));
	}
	
	/**
	 * Finds authority records by their authorized name
	 * @param authorizedName	the authorized name in un-normalized form
	 * @return					all authority records with the given authorized name
	 */
	public List<AuthorityRecord> findByRawAuthorizedName(String authorizedName)
	{
		return castList(
				helper.loadByStringQuery(
						"from " + AUTH_CLASS + " as auth where auth.authorizedForm.originalForm = ?",authorizedName));
	}
	
	/**
	 * Finds authority records by an alternate name
	 * @param alternateName		the alternate name, in un-normalized form
	 * @return					all authority records with the given alternate name
	 */
	public List<AuthorityRecord> findByRawAlternateName(String alternateName)
	{
		return castList(
				helper.loadByStringQuery(
						"from " + AUTH_CLASS + " as auth where auth.alternateForms.originalForm = ?",alternateName));
	}
	
	/**
	 * Finds the authority record with the given ID
	 * @param id		the ID to search for
	 * @return			an authority record with the given ID
	 * 
	 * @throws DataAccessException	if no record exists with the given ID
	 */
	public AuthorityRecord findById(Long id)
	throws DataAccessException
	{
		AuthorityRecord found = null;
		
		try
		{
			found = (AuthorityRecord) 
				helper.loadByID(
						AUTH_CLASS, id);
		}
		catch(Exception e)
		{
			throw new DataAccessException("Error loading record: " + e.getMessage());
		}
		
		if(found == null)
		{
			throw new DataAccessException("No record with id " + id);
		}
		
		return found;
	}
	
	protected List<AuthorityRecord> castList(List unsafe)
	{
		List<AuthorityRecord> safe = new LinkedList<AuthorityRecord>();
		Iterator unsafeItems = unsafe.iterator();
		
		while(unsafeItems.hasNext())
		{
			safe.add((AuthorityRecord) unsafeItems.next());
		}
		
		return safe;
	}
}