package info.extensiblecatalog.nacm.model.persistence;

import info.extensiblecatalog.nacm.model.AuthorityLink;
import info.extensiblecatalog.nacm.model.AuthorityRecord;
import info.extensiblecatalog.nacm.model.BibliographicRecord;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.hibernate.criterion.Restrictions;

/**
 * Provides store/find/delete functionality for
 * bibliographic records
 * @author Dean Rzonca
 *
 */
public class BibliographicRecordDAO
{
	static final String BIB_CLASS = BibliographicRecord.class.getName();
	
	HibernateHelper helper = new HibernateHelper();
	
	/**
	 * Stores a bibliographic record in the database
	 * If the ID is not set (-1), one will be set
	 * @param recordToStore	the bibliographic record to store
	 * @throws DataAccessException if the record cannot be stored
	 */
	public void store(BibliographicRecord recordToStore)
	throws DataAccessException
	{
		helper.store(recordToStore);
	}
	
	/**
	 * Removes a bibliographic record from the database
	 * @param id	the id of the bibliographic record to remove
	 */
	public void remove(BibliographicRecord record)
	throws DataAccessException
	{
		/*record.getAuthors().clear();
		record.getSubjects().clear();
		store(record);*/
		
		helper.remove(BIB_CLASS, record);
	}
	
	/**
	 * Finds bibliographic records by title
	 * @param title	the title to search for
	 * @return		All bibliographic records with the specified title
	 */
	public List<BibliographicRecord> findByNormalizedTitle(String title)
	{
		return castList(
				helper.loadByStringQuery(
						"from " + BIB_CLASS + " as bib where bib.titles.normalizedForm = ?",title));
	}
	
	/**
	 * Find bibliographic records by subject
	 * @param subject	the subject to search for, in normalized form
	 * @return			All bibliographic records with the specified subject
	 */
	public List<BibliographicRecord> findByNormalizedSubject(String subject)
	{
	return castList(
			helper.loadByStringQuery(
					"from " + BIB_CLASS + " as bib where bib.subjects.normalizedForm = ?",subject));
	}
	
	/**
	 * Find bibliographic records by author (as entered, not authorized)
	 * @param author	the author to search for, in normalized form
	 * @return			All bibliographic records with the specified author
	 */
	public List<BibliographicRecord> findByNormalizedAuthor(String author)
	{
		return castList(
				helper.loadByStringQuery(
						"from " + BIB_CLASS + " as bib where bib.authors.normalizedForm = ?",author));
	}
	
	/**
	 * Finds bibliographic records that correspond to a given authority record
	 * @param record	the authority record to search for
	 * @return			All bibliographic records that refer to the given authority record
	 */
	@SuppressWarnings("unchecked")
	public List<BibliographicRecord> findByAuthorityRecord(AuthorityRecord record)
	{
		List found = helper.getSession().createCriteria(BibliographicRecord.class)
		.createCriteria("authorizedAuthors")
			.createCriteria("record")
				.add(Restrictions.eq("Id", record.getId()))
				.list();
		
		found.addAll(helper.getSession().createCriteria(BibliographicRecord.class)
		.createCriteria("authorizedAuthors")
			.createCriteria("record")
				.add(Restrictions.eq("Id", record.getId()))
				.list());

		return castList(found);
	}
	
	/**
	 * Finds all bibliographic records that have NOT been searched against authority records
	 * @return			All unprocessed bibliographic records
	 */
	public List<BibliographicRecord> findAllUnmatched()
	{
		return castList(
				helper.loadByBooleanQuery(
						"from " + BIB_CLASS + " as bib where bib.matched = ?",false));
	}
	
	/**
	 * Finds all biblioraphic records that have been processed, have a level of
	 * confidence within a certain range, and have not been approved
	 * @param lowThreshold		the minimum confidence level, as a percent (inclusive)
	 * @param highThreshold		the maximum confidence level, as a percent (inclusive)
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<BibliographicRecord> findUnapprovedByConfidence(Integer lowThreshold, Integer highThreshold)
	{

		List found = helper.getSession().createCriteria(BibliographicRecord.class)
				.createCriteria("authorizedAuthors")
					.add(Restrictions.between("percentConfidence", lowThreshold, highThreshold) )
					.add(Restrictions.eq("status", AuthorityLink.NOT_REVIEWED) )
						.list();
		
		found.addAll(helper.getSession().createCriteria(BibliographicRecord.class)
		.createCriteria("authorizedAuthors")
			.add(Restrictions.between("percentConfidence", lowThreshold, highThreshold) )
			.add(Restrictions.eq("status", AuthorityLink.NOT_REVIEWED) )
				.list());
		
		return castList(found);
	}
	
	/**
	 * Finds the bibliographic record with the given ID
	 * @param id		the ID to search for
	 * @return			a bibliographic record
	 * 
	 * @throws DataAccessException	if no record exists with the given ID
	 */
	public BibliographicRecord findById(Long id)
	throws DataAccessException
	{
		BibliographicRecord found = null;
		
		try
		{
			found = (BibliographicRecord) 
				helper.loadByID(
						BIB_CLASS, id);
		}
		catch(Exception e)
		{
			throw new DataAccessException("Error loading record: " + e.getMessage());
		}
		
		if(found == null)
		{
			throw new DataAccessException("No record with id " + id);
		}
		
		return found;
	}
	
	/**
	 * Counts the records that have not been processed
	 * @return	the number of unprocessed records
	 */
	public int countUnmatchedRecords()
	{
		return helper.getSession().createQuery("from " + BIB_CLASS + " as bib where bib.matched = false").list().size();
	}
	
	protected List<BibliographicRecord> castList(List unsafe)
	{
		List<BibliographicRecord> safe = new LinkedList<BibliographicRecord>();
		Iterator unsafeItems = unsafe.iterator();
		
		while(unsafeItems.hasNext())
		{
			safe.add((BibliographicRecord) unsafeItems.next());
		}
		
		return safe;
	}
}