package info.extensiblecatalog.nacm.model.persistence;

import java.io.Serializable;
import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;

/**
 * Helper for interacting directly with Hibernate
 * This is only for internal use by the data persistence module,
 * hence the package-private visibility
 * @author Dean Rzonca
 *
 */
class HibernateHelper
{
	static final Long UNSAVED_ID = -1l;
	
	private static final SessionFactory sessionFactory;
	private static Session session;
	
	/**
	 * Stolen from Hibernate documentation
	 */
	static {
        try {
            // Create the SessionFactory from hibernate.cfg.xml
        	System.out.println("Creating session factory");
            sessionFactory = new Configuration().configure().buildSessionFactory();
            System.out.println("Success!");
            
            session = sessionFactory.openSession();
        } catch (Throwable ex) {
            // Make sure you log the exception, as it might be swallowed
            System.err.println("Initial SessionFactory creation failed." + ex);
            throw new ExceptionInInitializerError(ex);
        }
    }

	/**
	 * Gets the singleton SessionFactory
	 * A single SessionFactory is used since it is expensive
	 * to create
	 * @return	the singleton SessionFactory
	 */
    public static SessionFactory getSessionFactory() {
        return sessionFactory;
    }
    
    /**
     * Gets the active session
     * @return
     */
    public Session getSession()
    {
    	return session;
    }
    
    /**
     * Store a domain object
     * @param toStore	any mapped object
     */
    public void store(Object toStore)
    throws DataAccessException
    {
    	Transaction transaction = session.beginTransaction();
    	try
    	{
    		session.saveOrUpdate(toStore);
    		session.flush();
    	}
    	catch(Exception e)
    	{
    		transaction.rollback();
    		
    		// Wrap the actual exception
    		throw new DataAccessException("Could not store object: " + toStore, e);
    	}
    	transaction.commit();
    }
    
    /**
     * Gets a single object, by its unique ID
     * @param type		the fully-qualified Java type to load
     * @param id		the unique ID
     * @return			the object of type type with id id
     */
    public Object loadByID(String type, Serializable id)
    {
    	return session.get(type, id);
    }
    
    /**
     * Gets zero or more objects that match a query with one variable
     * @param query		the HQL query
     * @param value		the variable to be substituted
     * @return			all result objects
     */
    public List loadByStringQuery(String query, String value)
    {
    	Query crit = getSession().createQuery(query);
    	crit.setString(0, value);
    	
    	return crit.list();
    }
    
    /**
     * Gets zero or more objects that match a query with one variable
     * @param query		the HQL query
     * @param value		the variable to be substituted
     * @return			all result objects
     */
    public List loadByBooleanQuery(String query, Boolean value)
    {
    	Query crit = getSession().createQuery(query);
    	crit.setBoolean(0, value);
    	
    	return crit.list();
    }
    
    /**
     * Gets zero or more objects that match a query with one variable
     * @param query		the HQL query
     * @param value		the variable to be substituted
     * @return			all result objects
     */
    public List loadByIdQuery(String query, Long value)
    {
    	Query crit = getSession().createQuery(query);
    	crit.setLong(0, value);
    	
    	return crit.list();
    }
    
    /**
     * Remove an instance of an object from the database
     * @param type		the type of object to remove
     * @param toRemove	the instance to remove
     */
    public void remove(String type, Object toRemove)
    {
    	Transaction transaction = session.beginTransaction();
    	session.delete(type, toRemove);
    	session.flush();
    	transaction.commit();
    }
    
}