package info.extensiblecatalog.nacm.model.persistence.test;

import info.extensiblecatalog.nacm.model.AuthorityRecord;
import info.extensiblecatalog.nacm.model.Name;
import info.extensiblecatalog.nacm.model.persistence.AuthorityRecordDAO;
import info.extensiblecatalog.nacm.model.persistence.DataAccessException;

import java.util.List;
import java.util.Random;

import junit.framework.TestCase;

public class AuthorityRecordDAOTest extends TestCase
{
	private static final Long UNSAVED_VALUE = -1l;
	
	AuthorityRecordDAO dao;

	@Override
	protected void setUp() throws Exception
	{
		super.setUp();
		
		dao = new AuthorityRecordDAO();
	}

	public void testStore()
	throws DataAccessException
	{
		// Create a new authority record
		AuthorityRecord record = new AuthorityRecord();
		
		// Store it
		dao.store(record);
		
		Long newID = record.getId();
		
		// Make sure an ID gets assigned
		if(newID.equals(UNSAVED_VALUE))
		{
			fail("New ID was not properly assigned");
		}
		
		// Load the record
		AuthorityRecord found = dao.findById(newID);
		
		// Make sure they match
		assertEquals("Records do not match",record,found);
		
		// Clean up
		dao.remove(record);
	}

	public void testRemove()
	throws DataAccessException
	{
		// Create a new record
		AuthorityRecord record = new AuthorityRecord();
		
		// Store it
		dao.store(record);
		
		Long newID = record.getId();
		
		// Remove it
		dao.remove(record);
		
		// Try to load it now; this should cause an exception
		try
		{
			dao.findById(newID);
			fail("Should not be able to load a non-existant record");
		}
		catch(DataAccessException e)
		{
			// Nothing to do here
		}
	}

	public void testFindAllGeneratedRecords()
	throws DataAccessException
	{
		// Create a new authority record
		AuthorityRecord record = new AuthorityRecord();
		
		// Find out how many generated records there are now
		int count = dao.findAllGeneratedRecords().size();
		
		// Set the new record to generated
		record.setGenerated(true);
		
		// Store it
		dao.store(record);
		
		// Find out how many records there are now
		List<AuthorityRecord> found = dao.findAllGeneratedRecords();
		assertEquals("Number of records did not increase",count+1,found.size());
		
		// Make sure out record is in the results
		assertTrue("Record was not in results",found.contains(record));
		
		// Set it to not generated
		record.setGenerated(false);
		
		// Store it again
		dao.store(record);
		
		// Find out how many records there are now
		found = dao.findAllGeneratedRecords();
		assertEquals("Number of records did not decrease",count,found.size());
		
		// Clean up
		dao.remove(record);
	}

	public void testFindByAuthorizedName()
	throws DataAccessException
	{
		// Create a new authority record
		AuthorityRecord record = new AuthorityRecord();
		
		// Set the authorized name
		Name authorizedName = getRandomName();
		record.setAuthorizedForm(authorizedName);
		
		// Find out how many records have this name
		int count = dao.findByAuthorizedName(authorizedName.getNormalizedForm()).size();
		
		// Store the record
		dao.store(record);
		
		// Now, find out how many have the name
		List<AuthorityRecord> found = dao.findByAuthorizedName(authorizedName.getNormalizedForm());
		assertEquals("Number of records did not increase",count+1,found.size());
		
		// Make sure the record is in the results
		assertTrue("Results did not contain record",found.contains(record));
		
		// Clean up
		dao.remove(record);
	}

	public void testFindByAlternateName()
	throws DataAccessException
	{
		// Create a new authority record
		AuthorityRecord record = new AuthorityRecord();
		
		// Add an alternate form
		Name name1 = getRandomName();
		record.getAlternateForms().add(name1);
		
		// Find out how many records there are with this name
		int count1 = dao.findByAlternateName(name1.getNormalizedForm()).size();
		
		// Store the record
		dao.store(record);
		
		// Now find out how many there are
		List<AuthorityRecord> found = dao.findByAlternateName(name1.getNormalizedForm());
		assertEquals("Record count did not increase",count1+1,found.size());
		
		// Make sure record is in results
		assertTrue(found.contains(record));
		
		// Add another name
		Name name2 = getRandomName();
		record.getAlternateForms().add(name2);
		
		// Find out how many records have this name
		int count2 = dao.findByAlternateName(name2.getNormalizedForm()).size();
		
		// Store with both names
		dao.store(record);
		
		// Now find out how many records have the new name
		found = dao.findByAlternateName(name2.getNormalizedForm());
		assertEquals("Record count did not increase",count2+1,found.size());
		
		// Make sure the record is in the results
		assertTrue(found.contains(record));
		
		// Remove the first name
		record.getAlternateForms().remove(name1);
		
		// Store with name removed
		dao.store(record);
		
		// Find out how many records have the name now
		found = dao.findByAlternateName(name1.getNormalizedForm());
		assertEquals("Record count did not decrease",count1,found.size());
		
		// Clean up
		dao.remove(record);
	}

	public void testFindByRawAuthorizedName()
	throws DataAccessException
	{
		// Create a new authority record
		AuthorityRecord record = new AuthorityRecord();
		
		// Set the authorized name
		Name authorizedName = getRandomName();
		record.setAuthorizedForm(authorizedName);
		
		// Find out how many records have this name
		int count = dao.findByRawAuthorizedName(authorizedName.getOriginalForm()).size();
		
		// Store the record
		dao.store(record);
		
		// Now, find out how many have the name
		List<AuthorityRecord> found = dao.findByRawAuthorizedName(authorizedName.getOriginalForm());
		assertEquals("Number of records did not increase",count+1,found.size());
		
		// Make sure the record is in the results
		assertTrue("Results did not contain record",found.contains(record));
		
		// Clean up
		dao.remove(record);
	}

	public void testFindByRawAlternateName()
	throws DataAccessException
	{
		// Create a new authority record
		AuthorityRecord record = new AuthorityRecord();
		
		// Add an alternate form
		Name name1 = getRandomName();
		record.getAlternateForms().add(name1);
		
		// Find out how many records there are with this name
		int count1 = dao.findByRawAlternateName(name1.getOriginalForm()).size();
		
		// Store the record
		dao.store(record);
		
		// Now find out how many there are
		List<AuthorityRecord> found = dao.findByRawAlternateName(name1.getOriginalForm());
		assertEquals("Record count did not increase",count1+1,found.size());
		
		// Make sure record is in results
		assertTrue(found.contains(record));
		
		// Add another name
		Name name2 = getRandomName();
		record.getAlternateForms().add(name2);
		
		// Find out how many records have this name
		int count2 = dao.findByRawAlternateName(name2.getOriginalForm()).size();
		
		// Store with both names
		dao.store(record);
		
		// Now find out how many records have the new name
		found = dao.findByRawAlternateName(name2.getOriginalForm());
		assertEquals("Record count did not increase",count2+1,found.size());
		
		// Make sure the record is in the results
		assertTrue(found.contains(record));
		
		// Remove the first name
		record.getAlternateForms().remove(name1);
		
		// Store with name removed
		dao.store(record);
		
		// Find out how many records have the name now
		found = dao.findByAlternateName(name1.getNormalizedForm());
		assertEquals("Record count did not decrease",count1,found.size());
		
		// Clean up
		dao.remove(record);
	}

	public void testFindById()
	throws DataAccessException
	{
		// Create a new authority record
		AuthorityRecord record = new AuthorityRecord();
		
		// Store it
		dao.store(record);
		
		// Find it by ID
		AuthorityRecord found = dao.findById(record.getId());
		
		// Make sure it equals the original
		assertEquals("Records are not equal", record, found);
		
		// Remove the record
		dao.remove(record);
		
		// Try to find it by ID again
		try
		{
			found = dao.findById(record.getId());
			fail("Exception was not thrown when loading nonexistant record");
		}
		catch(DataAccessException e)
		{
			// Nothing to do here
		}
	}
	
	private Name getRandomName()
	{
		Name random = new Name();
		random.setNormalizedForm("normal" + getRandomString());
		random.setOriginalForm("original" + getRandomString());
	
		return random;
	}
	
	private String getRandomString()
	{
		return "Auth" + String.valueOf(new Random().nextInt());
	}

}