package info.extensiblecatalog.nacm.model.persistence.test;

import info.extensiblecatalog.nacm.model.AuthorityLink;
import info.extensiblecatalog.nacm.model.AuthorityRecord;
import info.extensiblecatalog.nacm.model.BibliographicRecord;
import info.extensiblecatalog.nacm.model.Name;
import info.extensiblecatalog.nacm.model.persistence.AuthorityRecordDAO;
import info.extensiblecatalog.nacm.model.persistence.BibliographicRecordDAO;
import info.extensiblecatalog.nacm.model.persistence.DataAccessException;

import java.util.Iterator;
import java.util.List;
import java.util.Random;

import junit.framework.TestCase;

public class BibliographicRecordDAOTest extends TestCase
{
	private static final Long UNSAVED_VALUE = -1l;

	BibliographicRecordDAO dao;
	
	@Override
	protected void setUp() throws Exception
	{
		super.setUp();
		
		dao = new BibliographicRecordDAO();
	}

	public void testStore()
	throws DataAccessException
	{	
		// Create a new record
		BibliographicRecord testRecord = new BibliographicRecord();
		testRecord.getAuthors().add(getRandomName());
		testRecord.getSubjects().add(getRandomName());
		testRecord.getTitles().add(getRandomName());
		testRecord.setOriginalXML(getRandomString());
		testRecord.setXMLHash(testRecord.getOriginalXML().hashCode());

		// Save it
		dao.store(testRecord);
		
		Long newID = testRecord.getId();
		
		// Storing the object for the first time should assign an ID
		if(newID.equals(UNSAVED_VALUE))
		{
			fail("New ID was not properly assigned");
		}
		
		// Load the record
		BibliographicRecord loadedRecord = dao.findById(newID);
		
		// Make sure they match
		assertEquals("Records do not match", testRecord, loadedRecord);
		
		// Clean up after self
		dao.remove(testRecord);
	}

	public void testRemove()
	throws DataAccessException
	{
		// Create a new record
		BibliographicRecord testRecord = new BibliographicRecord();
		
		// Store it
		dao.store(testRecord);
		
		Long newID = testRecord.getId();
		
		// Remove it
		dao.remove(testRecord);
		
		// Try to load it now; this should throw an exception
		try
		{
			dao.findById(newID);
			fail("Should not be able to load non-existant record by ID");
		}
		catch(DataAccessException e)
		{
			// nothing to do here
		}
	}

	public void testFindByTitle()
	throws DataAccessException
	{
		// Create a (hopefully) unique title
		Name testTitle = getRandomName();
		
		// Create a new record
		BibliographicRecord testRecord = new BibliographicRecord();
		
		// Add two titles
		testRecord.getTitles().add(getRandomName());
		
		// Store it
		dao.store(testRecord);
		
		// Change the title
		testRecord.getTitles().add(testTitle);
		
		// Find out how many records have this title now
		int recordCount = dao.findByNormalizedTitle(testTitle.getNormalizedForm()).size();
		
		// Store the record
		dao.store(testRecord);
		
		// Should be one more record with this title
		List<BibliographicRecord> records = dao.findByNormalizedTitle(testTitle.getNormalizedForm());
		assertEquals("Number of records incorrect",recordCount+1,records.size());
	
		// Make sure the added record is in the results
		assertTrue("Record was not in results", records.contains(testRecord));
		
		// Add another title
		testRecord.getTitles().add(getRandomName());
		
		// Clean up
		dao.remove(testRecord);
	}

	public void testFindByNormalizedSubject()
	throws DataAccessException
	{
		// Create a (hopefully) unique subject
		Name testSubject = getRandomName();
		
		// Create a new record
		BibliographicRecord testRecord = new BibliographicRecord();
		
		// Change the subject
		testRecord.getSubjects().add(testSubject);
		
		// Find out how many records have this subject now
		int recordCount = dao.findByNormalizedSubject(testSubject.getNormalizedForm()).size();
		
		// Store the record
		dao.store(testRecord);
		
		// Should be one more record with this subject
		List<BibliographicRecord> records = dao.findByNormalizedSubject(testSubject.getNormalizedForm());
		assertEquals("Number of records not correct", recordCount+1, records.size());
		
		// Make sure the record is in the results
		assertTrue("Record was not in results", records.contains(testRecord));
		
		// Remove the subject
		testRecord.getSubjects().clear();
		
		// Store it
		dao.store(testRecord);
		
		// Make sure the count went back down
		records = dao.findByNormalizedSubject(testSubject.getNormalizedForm());
		assertEquals("Number of records not correct",recordCount,records.size());
		
		// Clean up
		dao.remove(testRecord);
	}

	public void testFindByNormalizedAuthor()
	throws DataAccessException
	{
		// Create a (hopefully) unique normalized author
		Name testAuthor = getRandomName();
		
		// Create a new record
		BibliographicRecord testRecord = new BibliographicRecord();
		
		// Change the author
		testRecord.getAuthors().add(testAuthor);
		
		// Find out how many records have this author now
		int recordCount = dao.findByNormalizedAuthor(testAuthor.getNormalizedForm()).size();
		
		// Store the record
		dao.store(testRecord);
		
		// Should be one more record with this author
		List<BibliographicRecord> records = dao.findByNormalizedAuthor(testAuthor.getNormalizedForm());
		assertEquals("Number of records not correct", recordCount+1, records.size());
		
		// Make sure the record is in the results
		assertTrue("Record was not in results", records.contains(testRecord));
		
		// Clean up
		dao.remove(testRecord);
	}

	public void testFindByAuthorityRecord()
	throws DataAccessException
	{
		// Create a new record
		BibliographicRecord testRecord = new BibliographicRecord();
		
		// Get a random name
		Name testName = getRandomName();
		
		// Create a new Authority record
		AuthorityRecord authRecord = new AuthorityRecord();
		
		// Try to make them somewhat unique
		testRecord.getTitles().add(getRandomName());
		testRecord.getAuthors().add(testName);
		// Note that a bib record and an auth record cannot share the same instance of a Name
		authRecord.setAuthorizedForm(new Name(testName.getOriginalForm(),testName.getNormalizedForm()));
		
		// Link the records
		AuthorityLink link = new AuthorityLink();
		link.setRecord(authRecord);
		link.setRelatedName(testName);
		testRecord.getAuthorizedAuthors().add(link);
		
		// Store everything
		AuthorityRecordDAO authorityDAO = new AuthorityRecordDAO();
		authorityDAO.store(authRecord);
		dao.store(testRecord);
		
		// Find all bibliographic records with this authority record
		List<BibliographicRecord> found = dao.findByAuthorityRecord(authRecord);
		
		// Make sure our record is there
		assertTrue("Record was not in results",found.contains(testRecord));
		
		// Remove the association
		testRecord.getAuthorizedAuthors().clear();
		
		// Store the changes
		dao.store(testRecord);
		
		// Now, make sure our record isn't in there
		found = dao.findByAuthorityRecord(authRecord);
		
		assertFalse("Record was in results",found.contains(testRecord));
		
		// Clean up
		dao.remove(testRecord);
		authorityDAO.remove(authRecord);
	}

	public void testFindAllUnmatched()
	throws DataAccessException
	{
		// Create a new bibliographic record
		BibliographicRecord testRecord = new BibliographicRecord();
		
		// Try to make it unique
		testRecord.getTitles().add(getRandomName());
		
		// Find out how many unmatched records we have now
		int count = dao.findAllUnmatched().size();
		
		// Store the record
		dao.store(testRecord);
		
		// Now, find out how many records are unmatched
		List<BibliographicRecord> found = dao.findAllUnmatched();
		
		// Make sure it increased by one
		assertEquals("Number of records did not increase", count+1, found.size());
		
		// Make sure the record is in our results
		assertTrue("Record was not in results",found.contains(testRecord));
		
		// Mark the record as matched
		testRecord.setMatched(true);
		
		// Update it
		dao.store(testRecord);
		
		// Number of matched records should be down again
		assertEquals("Matched record appearing in unmatched results",count,dao.findAllUnmatched().size());
		
		// Clean up
		dao.remove(testRecord);
	}

	public void testFindUnapprovedByConfidence()
	throws DataAccessException
	{
		// Create a new bibliographic record
		BibliographicRecord testRecord = new BibliographicRecord();
		
		// Add an author
		Name author = getRandomName();
		testRecord.getAuthors().add(author);
		
		// Create a new authority record
		AuthorityRecord authRecord = new AuthorityRecord();
		
		// Store the authority record
		AuthorityRecordDAO authDAO = new AuthorityRecordDAO();
		authDAO.store(authRecord);
		
		// Create a link between the records with a confidence of 70%
		AuthorityLink link = new AuthorityLink();
		link.setPercentConfidence(70);
		link.setRecord(authRecord);
		link.setRelatedName(author);
		testRecord.getAuthorizedAuthors().add(link);
		
		// Store the bibliographic record
		dao.store(testRecord);
		
		// Check for the record using an all-inclusive query
		List<BibliographicRecord> found = dao.findUnapprovedByConfidence(0, 100);
		assertTrue("Record was not in results", found.contains(testRecord));
		
		// Check using a more precise query
		found = dao.findUnapprovedByConfidence(50, 80);
		assertTrue("Record was not in results", found.contains(testRecord));
		
		// Check using given percent as boundary
		found = dao.findUnapprovedByConfidence(70, 100);
		assertTrue("Record was not in results", found.contains(testRecord));
		
		// Make sure it does not appear when out of bounds
		found = dao.findUnapprovedByConfidence(50, 60);
		assertFalse("Out-of-bounds record in results",found.contains(testRecord));
		
		// Approve the match
		Iterator<AuthorityLink> iterator = testRecord.getAuthorizedAuthors().iterator();
		link = iterator.next();
		link.setStatus(AuthorityLink.MARKED_ACCEPTED);
		
		// Store the record with the match approved
		dao.store(testRecord);
		
		// findUnapprovedByConfidence should not find this, since it is approved
		found = dao.findUnapprovedByConfidence(60, 80);
		assertFalse("Approved match found when looking for unapproved", found.contains(testRecord));
		
		// Clean up
		dao.remove(testRecord);
		authDAO.remove(authRecord);
	}

	public void testFindById()
	throws DataAccessException
	{
		// Create a new bibliographic record
		BibliographicRecord testRecord = new BibliographicRecord();
		
		// Store it
		dao.store(testRecord);
		
		// Find it by ID
		BibliographicRecord found = dao.findById(testRecord.getId());
		
		// Make sure it equals the original
		assertEquals("Records are not equal", testRecord, found);
		
		// Remove the record
		dao.remove(testRecord);
		
		// Try to find it by ID again
		try
		{
			found = dao.findById(testRecord.getId());
			fail("Exception was not thrown when loading nonexistant record");
		}
		catch(DataAccessException e)
		{
			// Nothing to do here
		}
	}
	
	public void testCountUnmatchedRecords()
	throws DataAccessException
	{
		// Find out how many unmatched records there are now
		int count = dao.countUnmatchedRecords();
		
		// Create a new unmatched record
		BibliographicRecord record = new BibliographicRecord();
		
		// Set it to unmatched
		record.setMatched(false);
		
		// Store it
		dao.store(record);
		
		// Now find out how many unmatched records there are
		int newCount = dao.countUnmatchedRecords();
		
		// Make sure the number increased by one
		assertEquals("Unmatched count did not increase",count+1,newCount);
		
		// Now, set it to matched
		record.setMatched(true);
		
		// Store it
		dao.store(record);
		
		// Make sure the number went back down
		newCount = dao.countUnmatchedRecords();
		assertEquals("Unmatched count did not decrease",count,newCount);
		
		// Clean up
		dao.remove(record);
		
	}

	private Name getRandomName()
	{
		Name random = new Name();
		random.setNormalizedForm("normal" + getRandomString());
		random.setOriginalForm("original" + getRandomString());
	
		return random;
	}
	
	private String getRandomString()
	{
		return "Bib" + String.valueOf(new Random().nextInt());
	}
	
}
